@@ -39,6 +39,11 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
 /*--*/
 
 /* System library. */
@@ -70,6 +75,7 @@
 #include <rec_type.h>
 #include <cleanup_user.h>
 #include <tok822.h>
+#include <lex_822.h>
 #include <header_opts.h>
 #include <quote_822_local.h>
 #include <mail_params.h>
@@ -82,6 +88,8 @@
 #include <lex_822.h>
 #include <dsn_util.h>
 #include <conv_time.h>
+#include <info_log_addr_form.h>
+#include <hfrom_format.h>
 
 /* Application-specific. */
 
@@ -132,7 +140,8 @@ static char *cleanup_extract_internal(VSTRING *buffer, TOK822 *addr)
 
 /* cleanup_rewrite_sender - sender address rewriting */
 
-static void cleanup_rewrite_sender(CLEANUP_STATE *state, HEADER_OPTS *hdr_opts,
+static void cleanup_rewrite_sender(CLEANUP_STATE *state,
+				           const HEADER_OPTS *hdr_opts,
 				           VSTRING *header_buf)
 {
     TOK822 *tree;
@@ -168,7 +177,7 @@ static void cleanup_rewrite_sender(CLEANUP_STATE *state, HEADER_OPTS *hdr_opts,
 	    if (cleanup_masq_domains
 		&& (cleanup_masq_flags & CLEANUP_MASQ_FLAG_HDR_FROM))
 		did_rewrite |=
-		    cleanup_masquerade_tree(*tpp, cleanup_masq_domains);
+		    cleanup_masquerade_tree(state, *tpp, cleanup_masq_domains);
 	}
     }
     if (did_rewrite) {
@@ -176,7 +185,7 @@ static void cleanup_rewrite_sender(CLEANUP_STATE *state, HEADER_OPTS *hdr_opts,
 	vstring_strcat(header_buf, ": ");
 	tok822_externalize(header_buf, tree, TOK822_STR_HEAD);
     }
-    myfree((char *) addr_list);
+    myfree((void *) addr_list);
     tok822_free_tree(tree);
     if ((hdr_opts->flags & HDR_OPT_DROP) == 0) {
 	if (did_rewrite)
@@ -188,7 +197,8 @@ static void cleanup_rewrite_sender(CLEANUP_STATE *state, HEADER_OPTS *hdr_opts,
 
 /* cleanup_rewrite_recip - recipient address rewriting */
 
-static void cleanup_rewrite_recip(CLEANUP_STATE *state, HEADER_OPTS *hdr_opts,
+static void cleanup_rewrite_recip(CLEANUP_STATE *state,
+				          const HEADER_OPTS *hdr_opts,
 				          VSTRING *header_buf)
 {
     TOK822 *tree;
@@ -224,7 +234,7 @@ static void cleanup_rewrite_recip(CLEANUP_STATE *state, HEADER_OPTS *hdr_opts,
 	    if (cleanup_masq_domains
 		&& (cleanup_masq_flags & CLEANUP_MASQ_FLAG_HDR_RCPT))
 		did_rewrite |=
-		    cleanup_masquerade_tree(*tpp, cleanup_masq_domains);
+		    cleanup_masquerade_tree(state, *tpp, cleanup_masq_domains);
 	}
     }
     if (did_rewrite) {
@@ -232,7 +242,7 @@ static void cleanup_rewrite_recip(CLEANUP_STATE *state, HEADER_OPTS *hdr_opts,
 	vstring_strcat(header_buf, ": ");
 	tok822_externalize(header_buf, tree, TOK822_STR_HEAD);
     }
-    myfree((char *) addr_list);
+    myfree((void *) addr_list);
     tok822_free_tree(tree);
     if ((hdr_opts->flags & HDR_OPT_DROP) == 0) {
 	if (did_rewrite)
@@ -255,9 +265,11 @@ static void cleanup_act_log(CLEANUP_STATE *state,
     vstring_sprintf(state->temp1, "%s: %s: %s %.200s from %s;",
 		    state->queue_id, action, class, content, attr);
     if (state->sender)
-	vstring_sprintf_append(state->temp1, " from=<%s>", state->sender);
+	vstring_sprintf_append(state->temp1, " from=<%s>",
+			       info_log_addr_form_sender(state->sender));
     if (state->recip)
-	vstring_sprintf_append(state->temp1, " to=<%s>", state->recip);
+	vstring_sprintf_append(state->temp1, " to=<%s>",
+			       info_log_addr_form_recipient(state->recip));
     if ((attr = nvtable_find(state->attr, MAIL_ATTR_LOG_PROTO_NAME)) != 0)
 	vstring_sprintf_append(state->temp1, " proto=%s", attr);
     if ((attr = nvtable_find(state->attr, MAIL_ATTR_LOG_HELO_NAME)) != 0)
@@ -297,7 +309,7 @@ static const char *cleanup_act(CLEANUP_STATE *state, char *context,
      * queue record processing, and prevents bounces from being sent.
      */
     if (STREQUAL(value, "REJECT", command_len)) {
-	CLEANUP_STAT_DETAIL *detail;
+	const CLEANUP_STAT_DETAIL *detail;
 
 	if (state->reason)
 	    myfree(state->reason);
@@ -324,6 +336,10 @@ static const char *cleanup_act(CLEANUP_STATE *state, char *context,
 	cleanup_act_log(state, "warning", context, buf, optional_text);
 	return (buf);
     }
+    if (STREQUAL(value, "INFO", command_len)) {
+	cleanup_act_log(state, "info", context, buf, optional_text);
+	return (buf);
+    }
     if (STREQUAL(value, "FILTER", command_len)) {
 	if (*optional_text == 0) {
 	    msg_warn("missing FILTER command argument in %s map", map_class);
@@ -339,6 +355,11 @@ static const char *cleanup_act(CLEANUP_STATE *state, char *context,
 	}
 	return (buf);
     }
+    if (STREQUAL(value, "PASS", command_len)) {
+	cleanup_act_log(state, "pass", context, buf, optional_text);
+	state->flags &= ~CLEANUP_FLAG_FILTER_ALL;
+	return (buf);
+    }
     if (STREQUAL(value, "DISCARD", command_len)) {
 	cleanup_act_log(state, "discard", context, buf, optional_text);
 	state->flags |= CLEANUP_FLAG_DISCARD;
@@ -379,11 +400,26 @@ static const char *cleanup_act(CLEANUP_STATE *state, char *context,
     if (STREQUAL(value, "PREPEND", command_len)) {
 	if (*optional_text == 0) {
 	    msg_warn("PREPEND action without text in %s map", map_class);
-	} else if (strcmp(context, CLEANUP_ACT_CTXT_HEADER) == 0
-		   && !is_header(optional_text)) {
-	    msg_warn("bad PREPEND header text \"%s\" in %s map -- "
-		     "need \"headername: headervalue\"",
-		     optional_text, map_class);
+	} else if (strcmp(context, CLEANUP_ACT_CTXT_HEADER) == 0) {
+	    if (!is_header(optional_text)) {
+		msg_warn("bad PREPEND header text \"%s\" in %s map -- "
+			 "need \"headername: headervalue\"",
+			 optional_text, map_class);
+	    }
+
+	    /*
+	     * By design, cleanup_out_header() may modify content. Play safe
+	     * and prepare for future developments.
+	     */
+	    else {
+		VSTRING *temp;
+
+		cleanup_act_log(state, "prepend", context, buf, optional_text);
+		temp = vstring_strcpy(vstring_alloc(strlen(optional_text)),
+				      optional_text);
+		cleanup_out_header(state, temp);
+		vstring_free(temp);
+	    }
 	} else {
 	    cleanup_act_log(state, "prepend", context, buf, optional_text);
 	    cleanup_out_string(state, REC_TYPE_NORM, optional_text);
@@ -419,6 +455,23 @@ static const char *cleanup_act(CLEANUP_STATE *state, char *context,
 	}
 	return (buf);
     }
+    if (STREQUAL(value, "BCC", command_len)) {
+	if (strchr(optional_text, '@') == 0) {
+	    msg_warn("bad BCC address \"%s\" in %s map -- "
+		     "need user@domain",
+		     optional_text, map_class);
+	} else {
+	    if (state->hbc_rcpt == 0)
+		state->hbc_rcpt = argv_alloc(1);
+	    argv_add(state->hbc_rcpt, optional_text, (char *) 0);
+	    cleanup_act_log(state, "bcc", context, buf, optional_text);
+	}
+	return (buf);
+    }
+    if (STREQUAL(value, "STRIP", command_len)) {
+	cleanup_act_log(state, "strip", context, buf, optional_text);
+	return (CLEANUP_ACT_DROP);
+    }
     /* Allow and ignore optional text after the action. */
 
     if (STREQUAL(value, "IGNORE", command_len))
@@ -437,7 +490,8 @@ static const char *cleanup_act(CLEANUP_STATE *state, char *context,
 /* cleanup_header_callback - process one complete header line */
 
 static void cleanup_header_callback(void *context, int header_class,
-			         HEADER_OPTS *hdr_opts, VSTRING *header_buf,
+				            const HEADER_OPTS *hdr_opts,
+				            VSTRING *header_buf,
 				            off_t unused_offset)
 {
     CLEANUP_STATE *state = (CLEANUP_STATE *) context;
@@ -472,6 +526,10 @@ static void cleanup_header_callback(void *context, int header_class,
     if (hdr_opts && (hdr_opts->flags & HDR_OPT_MIME))
 	header_class = MIME_HDR_MULTIPART;
 
+    /* Update the Received: header count before maybe dropping headers below. */
+    if (hdr_opts && hdr_opts->type == HDR_RECEIVED)
+	state->hop_count += 1;
+
     if ((state->flags & CLEANUP_FLAG_FILTER)
 	&& (CHECK(MIME_HDR_PRIMARY, cleanup_header_checks, VAR_HEADER_CHECKS)
     || CHECK(MIME_HDR_MULTIPART, cleanup_mimehdr_checks, VAR_MIMEHDR_CHECKS)
@@ -489,8 +547,13 @@ static void cleanup_header_callback(void *context, int header_class,
 	    } else if (result != header) {
 		vstring_strcpy(header_buf, result);
 		hdr_opts = header_opts_find(result);
-		myfree((char *) result);
+		myfree((void *) result);
 	    }
+	} else if (checks->error) {
+	    msg_warn("%s: %s map lookup problem -- "
+		     "message not accepted, try again later",
+		     state->queue_id, checks->title);
+	    state->errs |= CLEANUP_STAT_WRITE;
 	}
     }
 
@@ -512,10 +575,11 @@ static void cleanup_header_callback(void *context, int header_class,
     while (ISSPACE(*hdrval))
 	hdrval++;
     /* trimblanks(hdrval, 0)[0] = 0; */
-    if (hdr_opts->type == HDR_CONTENT_TRANSFER_ENCODING) {
+    if (var_auto_8bit_enc_hdr
+	&& hdr_opts->type == HDR_CONTENT_TRANSFER_ENCODING) {
 	for (cmp = code_map; cmp->name != 0; cmp++) {
 	    if (strcasecmp(hdrval, cmp->name) == 0) {
-		if (strcmp(cmp->encoding, MAIL_ATTR_ENC_8BIT) == 0)
+		if (strcasecmp(cmp->encoding, MAIL_ATTR_ENC_8BIT) == 0)
 		    nvtable_update(state->attr, MAIL_ATTR_ENCODING,
 				   cmp->encoding);
 		break;
@@ -562,13 +626,26 @@ static void cleanup_header_callback(void *context, int header_class,
      */
     else {
 	state->headers_seen |= (1 << hdr_opts->type);
-	if (hdr_opts->type == HDR_MESSAGE_ID)
+	if (hdr_opts->type == HDR_MESSAGE_ID) {
+	    ssize_t len;
+
 	    msg_info("%s: message-id=%s", state->queue_id, hdrval);
+	    if (state->message_id == 0 && (len = balpar(hdrval, "<>")) > 0)
+		/* This Message ID may end up in threaded bounces. */
+		state->message_id = printable(mystrndup(hdrval, len), ' ');
+	}
 	if (hdr_opts->type == HDR_RESENT_MESSAGE_ID)
 	    msg_info("%s: resent-message-id=%s", state->queue_id, hdrval);
-	if (hdr_opts->type == HDR_RECEIVED)
-	    if (++state->hop_count >= var_hopcount_limit)
+	if (hdr_opts->type == HDR_RECEIVED) {
+	    if (state->hop_count >= var_hopcount_limit) {
+		msg_warn("%s: message rejected: hopcount exceeded",
+			 state->queue_id);
 		state->errs |= CLEANUP_STAT_HOPS;
+	    }
+	    /* Save our Received: header after maybe updating headers above. */
+	    if (state->hop_count == 1)
+		argv_add(state->auto_hdrs, vstring_str(header_buf), ARGV_END);
+	}
 	if (CLEANUP_OUT_OK(state)) {
 	    if (hdr_opts->flags & HDR_OPT_RR)
 		state->resent = "Resent-";
@@ -594,6 +671,7 @@ static void cleanup_header_done_callback(void *context)
     char    time_stamp[1024];		/* XXX locale dependent? */
     struct tm *tp;
     TOK822 *token;
+    TOK822 *dummy_token;
     time_t  tv;
 
     /*
@@ -606,6 +684,19 @@ static void cleanup_header_done_callback(void *context)
     if (CLEANUP_OUT_OK(state) == 0)
 	return;
 
+    /*
+     * Future proofing: the Milter client's header suppression algorithm
+     * assumes that the MTA prepends its own Received: header. This
+     * assumption may be violated after some source-code update. The
+     * following check ensures consistency, at least for local submission.
+     */
+    if (state->hop_count < 1) {
+	msg_warn("%s: message rejected: no Received: header",
+		 state->queue_id);
+	state->errs |= CLEANUP_STAT_BAD;
+	return;
+    }
+
     /*
      * Add a missing (Resent-)Message-Id: header. The message ID gives the
      * time in GMT units, plus the local queue ID.
@@ -620,25 +711,46 @@ static void cleanup_header_done_callback(void *context)
      * ID uniqueness only within a second, we must ensure that the time in
      * the message ID matches the queue ID creation time, as long as we use
      * the queue ID in the message ID.
+     * 
+     * XXX We log a dummy name=value record so that we (hopefully) don't break
+     * compatibility with existing logfile analyzers, and so that we don't
+     * complicate future code that wants to log more name=value attributes.
      */
-    if ((state->headers_seen & (1 << (state->resent[0] ?
+    if ((state->hdr_rewrite_context || var_always_add_hdrs)
+	&& (state->headers_seen & (1 << (state->resent[0] ?
 			   HDR_RESENT_MESSAGE_ID : HDR_MESSAGE_ID))) == 0) {
-	tv = state->handle->ctime.tv_sec;
-	tp = gmtime(&tv);
-	strftime(time_stamp, sizeof(time_stamp), "%Y%m%d%H%M%S", tp);
-	cleanup_out_format(state, REC_TYPE_NORM, "%sMessage-Id: <%s.%s@%s>",
-		state->resent, time_stamp, state->queue_id, var_myhostname);
-	msg_info("%s: %smessage-id=<%s.%s@%s>",
+	if (var_long_queue_ids) {
+	    vstring_sprintf(state->temp1, "%s@%s",
+			    state->queue_id, var_myhostname);
+	} else {
+	    tv = state->handle->ctime.tv_sec;
+	    tp = gmtime(&tv);
+	    strftime(time_stamp, sizeof(time_stamp), "%Y%m%d%H%M%S", tp);
+	    vstring_sprintf(state->temp1, "%s.%s@%s",
+			    time_stamp, state->queue_id, var_myhostname);
+	}
+	cleanup_out_format(state, REC_TYPE_NORM, "%sMessage-Id: <%s>",
+			   state->resent, vstring_str(state->temp1));
+	msg_info("%s: %smessage-id=<%s>",
 		 state->queue_id, *state->resent ? "resent-" : "",
-		 time_stamp, state->queue_id, var_myhostname);
+		 vstring_str(state->temp1));
+	state->headers_seen |= (1 << (state->resent[0] ?
+				   HDR_RESENT_MESSAGE_ID : HDR_MESSAGE_ID));
+	if (state->resent[0] == 0 && state->message_id == 0)
+	    state->message_id = concatenate("<", vstring_str(state->temp1),
+					    ">", (char *) 0);
+
     }
+    if ((state->headers_seen & (1 << HDR_MESSAGE_ID)) == 0)
+	msg_info("%s: message-id=<>", state->queue_id);
 
     /*
      * Add a missing (Resent-)Date: header. The date is in local time units,
      * with the GMT offset at the end.
      */
-    if ((state->headers_seen & (1 << (state->resent[0] ?
-				      HDR_RESENT_DATE : HDR_DATE))) == 0) {
+    if ((state->hdr_rewrite_context || var_always_add_hdrs)
+	&& (state->headers_seen & (1 << (state->resent[0] ?
+				       HDR_RESENT_DATE : HDR_DATE))) == 0) {
 	cleanup_out_format(state, REC_TYPE_NORM, "%sDate: %s",
 		      state->resent, mail_date(state->arrival_time.tv_sec));
     }
@@ -646,18 +758,62 @@ static void cleanup_header_done_callback(void *context)
     /*
      * Add a missing (Resent-)From: header.
      */
-    if ((state->headers_seen & (1 << (state->resent[0] ?
-				      HDR_RESENT_FROM : HDR_FROM))) == 0) {
+    if ((state->hdr_rewrite_context || var_always_add_hdrs)
+	&& (state->headers_seen & (1 << (state->resent[0] ?
+				       HDR_RESENT_FROM : HDR_FROM))) == 0) {
 	quote_822_local(state->temp1, *state->sender ?
 			state->sender : MAIL_ADDR_MAIL_DAEMON);
-	vstring_sprintf(state->temp2, "%sFrom: %s",
-			state->resent, vstring_str(state->temp1));
 	if (*state->sender && state->fullname && *state->fullname) {
-	    vstring_sprintf(state->temp1, "(%s)", state->fullname);
-	    token = tok822_parse(vstring_str(state->temp1));
-	    vstring_strcat(state->temp2, " ");
-	    tok822_externalize(state->temp2, token, TOK822_STR_NONE);
-	    tok822_free_tree(token);
+	    char   *cp;
+
+	    /* Enforce some sanity on full name content. */
+	    while ((cp = strchr(state->fullname, '\r')) != 0
+		   || (cp = strchr(state->fullname, '\n')) != 0)
+		*cp = ' ';
+
+	    /*
+	     * "From: phrase <route-addr>". Quote the phrase if it contains
+	     * specials or the "%!" legacy address operators.
+	     */
+	    if (cleanup_hfrom_format == HFROM_FORMAT_CODE_STD) {
+		vstring_sprintf(state->temp2, "%sFrom: ", state->resent);
+		if (state->fullname[strcspn(state->fullname,
+					    "%!" LEX_822_SPECIALS)] == 0) {
+		    /* Normalize whitespace. */
+		    token = tok822_scan_limit(state->fullname, &dummy_token,
+					      var_token_limit);
+		} else {
+		    token = tok822_alloc(TOK822_QSTRING, state->fullname);
+		}
+		if (token) {
+		    tok822_externalize(state->temp2, token, TOK822_STR_NONE);
+		    tok822_free(token);
+		    vstring_strcat(state->temp2, " ");
+		}
+		vstring_sprintf_append(state->temp2, "<%s>",
+				       vstring_str(state->temp1));
+	    }
+
+	    /*
+	     * "From: addr-spec (ctext)". This is the obsolete form.
+	     */
+	    else {
+		vstring_sprintf(state->temp2, "%sFrom: %s ",
+				state->resent, vstring_str(state->temp1));
+		vstring_sprintf(state->temp1, "(%s)", state->fullname);
+		token = tok822_parse(vstring_str(state->temp1));
+		tok822_externalize(state->temp2, token, TOK822_STR_NONE);
+		tok822_free_tree(token);
+	    }
+	}
+
+	/*
+	 * "From: addr-spec". This is the form in the absence of full name
+	 * information, also used for mail from mailer-daemon.
+	 */
+	else {
+	    vstring_sprintf(state->temp2, "%sFrom: %s",
+			    state->resent, vstring_str(state->temp1));
 	}
 	CLEANUP_OUT_BUF(state, REC_TYPE_NORM, state->temp2);
     }
@@ -691,8 +847,16 @@ static void cleanup_header_done_callback(void *context)
 #define VISIBLE_RCPT	((1 << HDR_TO) | (1 << HDR_RESENT_TO) \
 			| (1 << HDR_CC) | (1 << HDR_RESENT_CC))
 
-    if ((state->headers_seen & VISIBLE_RCPT) == 0)
-	cleanup_out_format(state, REC_TYPE_NORM, "%s", var_rcpt_witheld);
+    if ((state->hdr_rewrite_context || var_always_add_hdrs)
+	&& (state->headers_seen & VISIBLE_RCPT) == 0 && *var_rcpt_witheld) {
+	if (!is_header(var_rcpt_witheld)) {
+	    msg_warn("bad %s header text \"%s\" -- "
+		     "need \"headername: headervalue\"",
+		     VAR_RCPT_WITHELD, var_rcpt_witheld);
+	} else {
+	    cleanup_out_format(state, REC_TYPE_NORM, "%s", var_rcpt_witheld);
+	}
+    }
 
     /*
      * Place a dummy PTR record right after the last header so that we can
@@ -705,6 +869,7 @@ static void cleanup_header_done_callback(void *context)
 	cleanup_out_format(state, REC_TYPE_PTR, REC_TYPE_PTR_FORMAT, 0L);
 	if ((state->append_hdr_pt_target = vstream_ftell(state->dst)) < 0)
 	    msg_fatal("%s: vstream_ftell %s: %m", myname, cleanup_path);
+	state->body_offset = state->append_hdr_pt_target;
     }
 }
 
@@ -746,9 +911,14 @@ static void cleanup_body_callback(void *context, int type,
 		return;
 	    } else if (result != buf) {
 		cleanup_out(state, type, result, strlen(result));
-		myfree((char *) result);
+		myfree((void *) result);
 		return;
 	    }
+	} else if (cleanup_body_checks->error) {
+	    msg_warn("%s: %s map lookup problem -- "
+		     "message not accepted, try again later",
+		     state->queue_id, cleanup_body_checks->title);
+	    state->errs |= CLEANUP_STAT_WRITE;
 	}
     }
     cleanup_out(state, type, buf, len);
@@ -760,10 +930,27 @@ static void cleanup_message_headerbody(CLEANUP_STATE *state, int type,
 				               const char *buf, ssize_t len)
 {
     const char *myname = "cleanup_message_headerbody";
-    MIME_STATE_DETAIL *detail;
+    const MIME_STATE_DETAIL *detail;
     const char *cp;
     char   *dst;
 
+    /*
+     * Replace each stray CR or LF with one space. These are not allowed in
+     * SMTP, and can be used to enable outbound (remote) SMTP smuggling.
+     * Replacing these early ensures that our later DKIM etc. signature will
+     * not be invalidated. Besides preventing SMTP smuggling, replacing stray
+     * <CR> or <LF> ensures that the result of signature validation by a
+     * later mail system will not depend on how that mail system handles
+     * those stray characters in an implementation-dependent manner.
+     * 
+     * The input length is not changed, therefore it is safe to overwrite the
+     * input.
+     */
+    if (var_cleanup_mask_stray_cr_lf)
+	for (dst = (char *) buf; dst < buf + len; dst++)
+	    if (*dst == '\r' || *dst == '\n')
+		*dst = ' ';
+
     /*
      * Reject unwanted characters.
      * 
@@ -786,6 +973,8 @@ static void cleanup_message_headerbody(CLEANUP_STATE *state, int type,
     /*
      * Strip unwanted characters. Don't overwrite the input.
      * 
+     * XXX Possible space+time optimization: use a bitset.
+     * 
      * XXX Possible optimization: simplify the loop when the "strip" set
      * contains only one character.
      * 
@@ -831,6 +1020,7 @@ static void cleanup_message_headerbody(CLEANUP_STATE *state, int type,
 	state->mime_state = mime_state_free(state->mime_state);
 	if ((state->xtra_offset = vstream_ftell(state->dst)) < 0)
 	    msg_fatal("%s: vstream_ftell %s: %m", myname, cleanup_path);
+	state->cont_length = state->xtra_offset - state->data_offset;
 	state->action = cleanup_extracted;
     }
 
@@ -862,7 +1052,9 @@ static void cleanup_mime_error_callback(void *context, int err_code,
 #define TEXT_LEN (len < 100 ? (int) len : 100)
 	msg_info("%s: reject: mime-error %s: %.*s from %s; from=<%s> to=<%s>",
 		 state->queue_id, mime_state_error(err_code), TEXT_LEN, text,
-	    origin, state->sender, state->recip ? state->recip : "unknown");
+		 origin, info_log_addr_form_sender(state->sender),
+		 info_log_addr_form_recipient(state->recip ?
+					      state->recip : "unknown"));
     }
 }
 
@@ -887,6 +1079,9 @@ void    cleanup_message(CLEANUP_STATE *state, int type, const char *buf, ssize_t
      */
     mime_options = 0;
     if (var_disable_mime_input) {
+	if (var_force_mime_iconv)
+	    msg_fatal("do not specify both %s=yes and %s=yes",
+		      VAR_DISABLE_MIME_INPUT, VAR_FORCE_MIME_ICONV);
 	mime_options |= MIME_OPT_DISABLE_MIME;
     } else {
 	/* Turn off content checks if bouncing or forwarding mail. */
@@ -903,6 +1098,8 @@ void    cleanup_message(CLEANUP_STATE *state, int type, const char *buf, ssize_t
 		|| *var_nesthdr_checks)
 		mime_options |= MIME_OPT_REPORT_NESTING;
 	}
+	if (var_force_mime_iconv)
+	    mime_options |= MIME_OPT_DOWNGRADE;
     }
     state->mime_state = mime_state_alloc(mime_options,
 					 cleanup_header_callback,
@@ -914,7 +1111,7 @@ void    cleanup_message(CLEANUP_STATE *state, int type, const char *buf, ssize_t
 
     /*
      * XXX Workaround: truncate a long message header so that we don't exceed
-     * the Milter request size limit of 65535.
+     * the default Sendmail libmilter request size limit of 65535.
      */
 #define KLUDGE_HEADER_LIMIT	60000
     if ((cleanup_milters || state->milters)
